@@ -1,28 +1,29 @@
-/*
+/*-
  * SSLsplit - transparent SSL/TLS interception
- * Copyright (c) 2009-2016, Daniel Roethlisberger <daniel@roe.ch>
+ * https://www.roe.ch/SSLsplit
+ *
+ * Copyright (c) 2009-2018, Daniel Roethlisberger <daniel@roe.ch>.
  * All rights reserved.
- * http://www.roe.ch/SSLsplit
  *
  * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions, and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
+ * modification, are permitted provided that the following conditions are met:
+ * 1. Redistributions of source code must retain the above copyright notice,
+ *    this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright notice,
+ *    this list of conditions and the following disclaimer in the documentation
+ *    and/or other materials provided with the distribution.
  *
- * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
- * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
- * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
- * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
- * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
- * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER AND CONTRIBUTORS ``AS IS''
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
  */
 
 #include "pxyconn.h"
@@ -118,16 +119,19 @@ typedef struct pxy_conn_ctx {
 	struct pxy_conn_desc dst;
 
 	/* status flags */
+	unsigned int connected : 1;       /* 0 until both ends are connected */
+	unsigned int enomem : 1;                       /* 1 if out of memory */
+	/* ssl */
+	unsigned int sni_peek_retries : 6;       /* max 64 SNI parse retries */
 	unsigned int immutable_cert : 1;  /* 1 if the cert cannot be changed */
 	unsigned int generated_cert : 1;     /* 1 if we generated a new cert */
-	unsigned int connected : 1;       /* 0 until both ends are connected */
+	unsigned int passthrough : 1;      /* 1 if SSL passthrough is active */
+	/* http */
 	unsigned int seen_req_header : 1; /* 0 until request header complete */
 	unsigned int seen_resp_header : 1;  /* 0 until response hdr complete */
 	unsigned int sent_http_conn_close : 1;   /* 0 until Conn: close sent */
-	unsigned int passthrough : 1;      /* 1 if SSL passthrough is active */
 	unsigned int ocsp_denied : 1;                /* 1 if OCSP was denied */
-	unsigned int enomem : 1;                       /* 1 if out of memory */
-	unsigned int sni_peek_retries : 6;       /* max 64 SNI parse retries */
+	/* autossl */
 	unsigned int clienthello_search : 1;       /* 1 if waiting for hello */
 	unsigned int clienthello_found : 1;      /* 1 if conn upgrade to SSL */
 
@@ -217,7 +221,7 @@ pxy_conn_ctx_new(proxyspec_t *spec, opts_t *opts,
 }
 
 static void NONNULL(1)
-pxy_conn_ctx_free(pxy_conn_ctx_t *ctx)
+pxy_conn_ctx_free(pxy_conn_ctx_t *ctx, int by_requestor)
 {
 #ifdef DEBUG_PROXY
 	if (OPTS_DEBUG(ctx->opts)) {
@@ -225,6 +229,11 @@ pxy_conn_ctx_free(pxy_conn_ctx_t *ctx)
 		                (void*)ctx);
 	}
 #endif /* DEBUG_PROXY */
+	if (WANT_CONTENT_LOG(ctx) && ctx->logctx) {
+		if (log_content_close(&ctx->logctx, by_requestor) == -1) {
+			log_err_printf("Warning: Content log close failed\n");
+		}
+	}
 	pxy_thrmgr_detach(ctx->thrmgr, ctx->thridx);
 	if (ctx->srchost_str) {
 		free(ctx->srchost_str);
@@ -288,11 +297,6 @@ pxy_conn_ctx_free(pxy_conn_ctx_t *ctx)
 	if (ctx->sni) {
 		free(ctx->sni);
 	}
-	if (WANT_CONTENT_LOG(ctx) && ctx->logctx) {
-		if (log_content_close(&ctx->logctx) == -1) {
-			log_err_printf("Warning: Content log close failed\n");
-		}
-	}
 	free(ctx);
 }
 
@@ -309,7 +313,12 @@ static int pxy_ossl_servername_cb(SSL *ssl, int *al, void *arg);
 #endif /* !OPENSSL_NO_TLSEXT */
 static int pxy_ossl_sessnew_cb(SSL *, SSL_SESSION *);
 static void pxy_ossl_sessremove_cb(SSL_CTX *, SSL_SESSION *);
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
 static SSL_SESSION * pxy_ossl_sessget_cb(SSL *, unsigned char *, int, int *);
+#else /* OPENSSL_VERSION_NUMBER >= 0x10100000L */
+static SSL_SESSION * pxy_ossl_sessget_cb(SSL *, const unsigned char *, int,
+                                         int *);
+#endif /* OPENSSL_VERSION_NUMBER >= 0x10100000L */
 
 /*
  * Dump information on a certificate to the debug log.
@@ -369,6 +378,12 @@ pxy_log_connect_nonhttp(pxy_conn_ctx_t *ctx)
 	}
 #endif /* HAVE_LOCAL_PROCINFO */
 
+	/*
+	 * The following ifdef's within asprintf arguments list generates
+	 * warnings with -Wembedded-directive on some compilers.
+	 * Not fixing the code in order to avoid more code duplication.
+	 */
+
 	if (!ctx->src.ssl) {
 		rv = asprintf(&msg, "%s %s %s %s %s"
 #ifdef HAVE_LOCAL_PROCINFO
@@ -466,6 +481,12 @@ pxy_log_connect_http(pxy_conn_ctx_t *ctx)
 	}
 #endif /* HAVE_LOCAL_PROCINFO */
 
+	/*
+	 * The following ifdef's within asprintf arguments list generates
+	 * warnings with -Wembedded-directive on some compilers.
+	 * Not fixing the code in order to avoid more code duplication.
+	 */
+
 	if (!ctx->spec->ssl) {
 		rv = asprintf(&msg, "http %s %s %s %s %s %s %s %s %s"
 #ifdef HAVE_LOCAL_PROCINFO
@@ -604,7 +625,13 @@ pxy_ossl_sessremove_cb(UNUSED SSL_CTX *sslctx, SSL_SESSION *sess)
  * Called by OpenSSL when a src SSL session is requested by the client.
  */
 static SSL_SESSION *
-pxy_ossl_sessget_cb(UNUSED SSL *ssl, unsigned char *id, int idlen, int *copy)
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
+pxy_ossl_sessget_cb(UNUSED SSL *ssl, unsigned char *id, int idlen,
+                    int *copy)
+#else /* OPENSSL_VERSION_NUMBER >= 0x10100000L */
+pxy_ossl_sessget_cb(UNUSED SSL *ssl, const unsigned char *id, int idlen,
+                    int *copy)
+#endif /* OPENSSL_VERSION_NUMBER >= 0x10100000L */
 {
 	SSL_SESSION *sess;
 
@@ -645,18 +672,14 @@ pxy_sslctx_setoptions(SSL_CTX *sslctx, pxy_conn_ctx_t *ctx)
 	SSL_CTX_set_options(sslctx, SSL_OP_NO_TICKET);
 #endif /* SSL_OP_NO_TICKET */
 
-	/*
-	 * Do not use HAVE_SSLV2 because we need to set SSL_OP_NO_SSLv2 if it
-	 * is available and WITH_SSLV2 was not used.
-	 */
 #ifdef SSL_OP_NO_SSLv2
-#ifdef WITH_SSLV2
+#ifdef HAVE_SSLV2
 	if (ctx->opts->no_ssl2) {
-#endif /* WITH_SSLV2 */
+#endif /* HAVE_SSLV2 */
 		SSL_CTX_set_options(sslctx, SSL_OP_NO_SSLv2);
-#ifdef WITH_SSLV2
+#ifdef HAVE_SSLV2
 	}
-#endif /* WITH_SSLV2 */
+#endif /* HAVE_SSLV2 */
 #endif /* !SSL_OP_NO_SSLv2 */
 #ifdef HAVE_SSLV3
 	if (ctx->opts->no_ssl3) {
@@ -871,8 +894,10 @@ pxy_srccert_create(pxy_conn_ctx_t *ctx)
 				log_dbg_printf("Certificate cache: MISS\n");
 			cert->crt = ssl_x509_forge(ctx->opts->cacrt,
 			                           ctx->opts->cakey,
-			                           ctx->origcrt, NULL,
-			                           ctx->opts->key);
+			                           ctx->origcrt,
+			                           ctx->opts->key,
+			                           NULL,
+			                           ctx->opts->crlurl);
 			cachemgr_fkcrt_set(ctx->origcrt, cert->crt);
 		}
 		cert_set_key(cert, ctx->opts->key);
@@ -1019,7 +1044,8 @@ pxy_ossl_servername_cb(SSL *ssl, UNUSED int *al, void *arg)
 			               "(SNI mismatch)\n");
 		}
 		newcrt = ssl_x509_forge(ctx->opts->cacrt, ctx->opts->cakey,
-		                        sslcrt, sn, ctx->opts->key);
+		                        sslcrt, ctx->opts->key,
+		                        sn, ctx->opts->crlurl);
 		if (!newcrt) {
 			ctx->enomem = 1;
 			return SSL_TLSEXT_ERR_NOACK;
@@ -1131,7 +1157,7 @@ bufferevent_free_and_close_fd(struct bufferevent *bev, pxy_conn_ctx_t *ctx)
 	evutil_socket_t fd = bufferevent_getfd(bev);
 	SSL *ssl = NULL;
 
-	if (ctx->spec->ssl && !ctx->passthrough) {
+	if ((ctx->spec->ssl || ctx->clienthello_found) && !ctx->passthrough) {
 		ssl = bufferevent_openssl_get_ssl(bev); /* does not inc refc */
 	}
 
@@ -1260,6 +1286,7 @@ pxy_http_reqhdr_filter_line(const char *line, pxy_conn_ctx_t *ctx)
 				ctx->enomem = 1;
 				return NULL;
 			}
+		/* Override Connection: keepalive and Connection: upgrade */
 		} else if (!strncasecmp(line, "Connection:", 11)) {
 			ctx->sent_http_conn_close = 1;
 			if (!(newhdr = strdup("Connection: close"))) {
@@ -1267,7 +1294,10 @@ pxy_http_reqhdr_filter_line(const char *line, pxy_conn_ctx_t *ctx)
 				return NULL;
 			}
 			return newhdr;
-		} else if (!strncasecmp(line, "Accept-Encoding:", 16) ||
+		/* Suppress upgrading to SSL/TLS, WebSockets or HTTP/2,
+		 * unsupported encodings, and keep-alive */
+		} else if (!strncasecmp(line, "Upgrade:", 8) ||
+		           !strncasecmp(line, "Accept-Encoding:", 16) ||
 		           !strncasecmp(line, "Keep-Alive:", 11)) {
 			return NULL;
 		} else if (line[0] == '\0') {
@@ -1351,7 +1381,11 @@ pxy_http_resphdr_filter_line(const char *line, pxy_conn_ctx_t *ctx)
 		    !strncasecmp(line, "Strict-Transport-Security:", 26) ||
 		    /* Alternate Protocol
 		     * remove to prevent switching to QUIC, SPDY et al */
-		    !strncasecmp(line, "Alternate-Protocol:", 19)) {
+		    !strncasecmp(line, "Alternate-Protocol:", 19) ||
+		    /* Upgrade header
+		     * remove to prevent upgrading to HTTPS in unhandled ways,
+		     * and more importantly, WebSockets and HTTP/2 */
+		    !strncasecmp(line, "Upgrade:", 8)) {
 			return NULL;
 		} else if (line[0] == '\0') {
 			ctx->seen_resp_header = 1;
@@ -1527,17 +1561,19 @@ pxy_conn_autossl_peek_and_upgrade(pxy_conn_ctx_t *ctx)
 			}
 			ctx->dst.bev = bufferevent_openssl_filter_new(
 			               ctx->evbase, ctx->dst.bev, ctx->dst.ssl,
-			               BUFFEREVENT_SSL_CONNECTING, 0);
+			               BUFFEREVENT_SSL_CONNECTING,
+			               BEV_OPT_DEFER_CALLBACKS);
+			if (!ctx->dst.bev) {
+				return 0;
+			}
 			bufferevent_setcb(ctx->dst.bev, pxy_bev_readcb,
 			                  pxy_bev_writecb, pxy_bev_eventcb,
 			                  ctx);
 			bufferevent_enable(ctx->dst.bev, EV_READ|EV_WRITE);
-			if(!ctx->dst.bev) {
-				return 0;
-			}
-			if( OPTS_DEBUG(ctx->opts)) {
+			if (OPTS_DEBUG(ctx->opts)) {
 				log_err_printf("Replaced dst bufferevent, new "
-				               "one is %p\n", ctx->dst.bev);
+				               "one is %p\n",
+				               (void*)ctx->dst.bev);
 			}
 			ctx->clienthello_search = 0;
 			ctx->clienthello_found = 1;
@@ -1552,8 +1588,8 @@ pxy_conn_autossl_peek_and_upgrade(pxy_conn_ctx_t *ctx)
 	return 0;
 }
 
-void
-pxy_conn_terminate_free(pxy_conn_ctx_t *ctx)
+static void
+pxy_conn_terminate_free(pxy_conn_ctx_t *ctx, int is_requestor)
 {
 	log_err_printf("Terminating connection%s!\n",
 	               ctx->enomem ? " (out of memory)" : "");
@@ -1565,7 +1601,7 @@ pxy_conn_terminate_free(pxy_conn_ctx_t *ctx)
 		bufferevent_free_and_close_fd(ctx->src.bev, ctx);
 		ctx->src.bev = NULL;
 	}
-	pxy_conn_ctx_free(ctx);
+	pxy_conn_ctx_free(ctx, is_requestor);
 }
 
 /*
@@ -1588,9 +1624,8 @@ pxy_bev_readcb(struct bufferevent *bev, void *arg)
 	if (!ctx->connected) {
 		log_err_printf("readcb called when other end not connected - "
 		               "aborting.\n");
-		/* XXX should signal main loop instead of calling exit() */
-		log_fini();
-		exit(EXIT_FAILURE);
+		log_exceptcb();
+		return;
 	}
 
 	if (ctx->clienthello_search) {
@@ -1601,6 +1636,8 @@ pxy_bev_readcb(struct bufferevent *bev, void *arg)
 
 	struct evbuffer *inbuf = bufferevent_get_input(bev);
 	if (other->closed) {
+		log_dbg_printf("Warning: Drained %zu bytes (conn closed)\n",
+		               evbuffer_get_length(inbuf));
 		evbuffer_drain(inbuf, evbuffer_get_length(inbuf));
 		return;
 	}
@@ -1706,7 +1743,7 @@ pxy_bev_readcb(struct bufferevent *bev, void *arg)
 
 	/* out of memory condition? */
 	if (ctx->enomem) {
-		pxy_conn_terminate_free(ctx);
+		pxy_conn_terminate_free(ctx, (bev == ctx->src.bev));
 		return;
 	}
 
@@ -1760,7 +1797,7 @@ pxy_bev_writecb(struct bufferevent *bev, void *arg)
 			/* finished writing and other end is closed;
 			 * close this end too and clean up memory */
 			bufferevent_free_and_close_fd(bev, ctx);
-			pxy_conn_ctx_free(ctx);
+			pxy_conn_ctx_free(ctx, (bev == ctx->dst.bev));
 		}
 		return;
 	}
@@ -1783,6 +1820,7 @@ pxy_bev_eventcb(struct bufferevent *bev, short events, void *arg)
 	pxy_conn_ctx_t *ctx = arg;
 	pxy_conn_desc_t *this = (bev==ctx->src.bev) ? &ctx->src : &ctx->dst;
 	pxy_conn_desc_t *other = (bev==ctx->src.bev) ? &ctx->dst : &ctx->src;
+	int is_requestor = (bev == ctx->src.bev);
 
 #ifdef DEBUG_PROXY
 	if (OPTS_DEBUG(ctx->opts)) {
@@ -1827,7 +1865,7 @@ pxy_bev_eventcb(struct bufferevent *bev, short events, void *arg)
 					return;
 				}
 				evutil_closesocket(ctx->fd);
-				pxy_conn_ctx_free(ctx);
+				pxy_conn_ctx_free(ctx, 1);
 				return;
 			}
 		}
@@ -1839,10 +1877,15 @@ pxy_bev_eventcb(struct bufferevent *bev, short events, void *arg)
 			               ctx->evbase, ctx->src.bev, ctx->src.ssl,
 			               BUFFEREVENT_SSL_ACCEPTING,
 			               BEV_OPT_DEFER_CALLBACKS);
-			bufferevent_setcb(ctx->src.bev, pxy_bev_readcb,
-			                  pxy_bev_writecb, pxy_bev_eventcb,
-			                  ctx);
-			bufferevent_enable(ctx->src.bev, EV_READ|EV_WRITE);
+			if (ctx->src.bev) {
+				bufferevent_setcb(ctx->src.bev,
+				                  pxy_bev_readcb,
+				                  pxy_bev_writecb,
+				                  pxy_bev_eventcb,
+				                  ctx);
+				bufferevent_enable(ctx->src.bev,
+				                   EV_READ|EV_WRITE);
+			}
 		} else {
 			ctx->src.bev = pxy_bufferevent_setup(ctx, ctx->fd,
 			                                     ctx->src.ssl);
@@ -1854,7 +1897,7 @@ pxy_bev_eventcb(struct bufferevent *bev, short events, void *arg)
 			}
 			bufferevent_free_and_close_fd(bev, ctx);
 			evutil_closesocket(ctx->fd);
-			pxy_conn_ctx_free(ctx);
+			pxy_conn_ctx_free(ctx, 1);
 			return;
 		}
 
@@ -1865,7 +1908,7 @@ pxy_bev_eventcb(struct bufferevent *bev, short events, void *arg)
 			                     &ctx->dsthost_str,
 			                     &ctx->dstport_str) != 0) {
 				ctx->enomem = 1;
-				pxy_conn_terminate_free(ctx);
+				pxy_conn_terminate_free(ctx, 1);
 				return;
 			}
 
@@ -1888,7 +1931,7 @@ pxy_bev_eventcb(struct bufferevent *bev, short events, void *arg)
 					if (!ctx->lproc.user ||
 					    !ctx->lproc.group) {
 						ctx->enomem = 1;
-						pxy_conn_terminate_free(ctx);
+						pxy_conn_terminate_free(ctx, 1);
 						return;
 					}
 				}
@@ -1909,7 +1952,7 @@ pxy_bev_eventcb(struct bufferevent *bev, short events, void *arg)
 			                    ) == -1) {
 				if (errno == ENOMEM)
 					ctx->enomem = 1;
-				pxy_conn_terminate_free(ctx);
+				pxy_conn_terminate_free(ctx, 1);
 				return;
 			}
 		}
@@ -1922,14 +1965,29 @@ connected:
 			pxy_log_connect_nonhttp(ctx);
 		}
 
-		/* write SSL certificates to gendir */
-		if (this->ssl && (bev == ctx->src.bev) &&
-		    ctx->opts->certgendir) {
-			pxy_srccert_write(ctx);
+		if (this->ssl) {
+			/* write SSL certificates to gendir */
+			if ((bev == ctx->src.bev) && ctx->opts->certgendir) {
+				pxy_srccert_write(ctx);
+			}
+
+			/* log master key */
+			if (ctx->opts->masterkeylog) {
+				char *keystr;
+				keystr = ssl_ssl_masterkey_to_str(this->ssl);
+				if ((keystr == NULL) ||
+				    (log_masterkey_print_free(keystr) == -1)) {
+					if (errno == ENOMEM)
+						ctx->enomem = 1;
+					pxy_conn_terminate_free(ctx, 1);
+					return;
+				}
+			}
 		}
 
 		if (OPTS_DEBUG(ctx->opts)) {
 			if (this->ssl) {
+				char *keystr;
 				/* for SSL, we get two connect events */
 				log_dbg_printf("SSL connected %s [%s]:%s"
 				               " %s %s\n",
@@ -1943,6 +2001,10 @@ connected:
 				               ctx->srcport_str,
 				               SSL_get_version(this->ssl),
 				               SSL_get_cipher(this->ssl));
+				keystr = ssl_ssl_masterkey_to_str(this->ssl);
+				if (keystr) {
+					log_dbg_print_free(keystr);
+				}
 			} else {
 				/* for TCP, we get only a dst connect event,
 				 * since src was already connected from the
@@ -1986,8 +2048,9 @@ connected:
 		           SSL_R_SSLV3_ALERT_HANDSHAKE_FAILURE) {
 			/* these can happen due to client cert auth,
 			 * only log error if debugging is activated */
-			log_dbg_printf("Error from bufferevent: "
+			log_dbg_printf("Error from %s bufferevent: "
 			               "%i:%s %lu:%i:%s:%i:%s:%i:%s\n",
+			               (bev == ctx->src.bev) ? "src" : "dst",
 			               errno,
 			               errno ? strerror(errno) : "-",
 			               sslerr,
@@ -2013,8 +2076,9 @@ connected:
 			}
 		} else {
 			/* real errors */
-			log_err_printf("Error from bufferevent: "
+			log_err_printf("Error from %s bufferevent: "
 			               "%i:%s %lu:%i:%s:%i:%s:%i:%s\n",
+			               (bev == ctx->src.bev) ? "src" : "dst",
 			               errno,
 			               errno ? strerror(errno) : "-",
 			               sslerr,
@@ -2075,28 +2139,43 @@ connected:
 	}
 
 	if (events & BEV_EVENT_EOF) {
+#ifdef DEBUG_PROXY
+		if (OPTS_DEBUG(ctx->opts)) {
+			log_dbg_printf("evbuffer size at EOF: "
+			               "i:%zu o:%zu i:%zu o:%zu\n",
+			                evbuffer_get_length(
+			                    bufferevent_get_input(bev)),
+			                evbuffer_get_length(
+			                    bufferevent_get_output(bev)),
+			                evbuffer_get_length(
+			                    other->closed ? 0 :
+			                    bufferevent_get_input(other->bev)),
+			                evbuffer_get_length(
+			                    other->closed ? 0 :
+			                    bufferevent_get_output(other->bev))
+			                );
+		}
+#endif /* DEBUG_PROXY */
 		if (!ctx->connected) {
-			log_dbg_printf("EOF on inbound connection while "
-			               "connecting to original destination\n");
+			log_dbg_printf("EOF on outbound connection before "
+			               "connection establishment\n");
 			evutil_closesocket(ctx->fd);
 			other->closed = 1;
 		} else if (!other->closed) {
-			struct evbuffer *inbuf, *outbuf;
-			inbuf = bufferevent_get_input(bev);
-			outbuf = bufferevent_get_output(other->bev);
-			if (evbuffer_get_length(inbuf) > 0) {
-				evbuffer_add_buffer(outbuf, inbuf);
-			} else {
-				/* if the other end is still open and doesn't
-				 * have data to send, close it, otherwise its
-				 * writecb will close it after writing what's
-				 * left in the output buffer. */
-				if (evbuffer_get_length(outbuf) == 0) {
-					bufferevent_free_and_close_fd(
-							other->bev, ctx);
-					other->bev = NULL;
-					other->closed = 1;
-				}
+			/* if there is data pending in the closed connection,
+			 * handle it here, otherwise it will be lost. */
+			if (evbuffer_get_length(bufferevent_get_input(bev))) {
+				pxy_bev_readcb(bev, ctx);
+			}
+			/* if the other end is still open and doesn't
+			 * have data to send, close it, otherwise its
+			 * writecb will close it after writing what's
+			 * left in the output buffer. */
+			if (evbuffer_get_length(
+			    bufferevent_get_output(other->bev)) == 0) {
+				bufferevent_free_and_close_fd(other->bev, ctx);
+				other->bev = NULL;
+				other->closed = 1;
 			}
 		}
 		goto leave;
@@ -2120,7 +2199,7 @@ leave:
 	bufferevent_free_and_close_fd(bev, ctx);
 	this->bev = NULL;
 	if (other->closed) {
-		pxy_conn_ctx_free(ctx);
+		pxy_conn_ctx_free(ctx, is_requestor);
 	}
 }
 
@@ -2134,7 +2213,7 @@ pxy_conn_connect(pxy_conn_ctx_t *ctx)
 	if (!ctx->addrlen) {
 		log_err_printf("No target address; aborting connection\n");
 		evutil_closesocket(ctx->fd);
-		pxy_conn_ctx_free(ctx);
+		pxy_conn_ctx_free(ctx, 1);
 		return;
 	}
 
@@ -2144,7 +2223,7 @@ pxy_conn_connect(pxy_conn_ctx_t *ctx)
 		if (!ctx->dst.ssl) {
 			log_err_printf("Error creating SSL\n");
 			evutil_closesocket(ctx->fd);
-			pxy_conn_ctx_free(ctx);
+			pxy_conn_ctx_free(ctx, 1);
 			return;
 		}
 	}
@@ -2155,7 +2234,7 @@ pxy_conn_connect(pxy_conn_ctx_t *ctx)
 			ctx->dst.ssl = NULL;
 		}
 		evutil_closesocket(ctx->fd);
-		pxy_conn_ctx_free(ctx);
+		pxy_conn_ctx_free(ctx, 1);
 		return;
 	}
 
@@ -2191,7 +2270,7 @@ pxy_sni_resolve_cb(int errcode, struct evutil_addrinfo *ai, void *arg)
 		log_err_printf("Cannot resolve SNI hostname '%s': %s\n",
 		               ctx->sni, evutil_gai_strerror(errcode));
 		evutil_closesocket(ctx->fd);
-		pxy_conn_ctx_free(ctx);
+		pxy_conn_ctx_free(ctx, 1);
 		return;
 	}
 
@@ -2232,13 +2311,13 @@ pxy_fd_readcb(MAYBE_UNUSED evutil_socket_t fd, UNUSED short what, void *arg)
 			log_err_printf("Error peeking on fd, aborting "
 			               "connection\n");
 			evutil_closesocket(fd);
-			pxy_conn_ctx_free(ctx);
+			pxy_conn_ctx_free(ctx, 1);
 			return;
 		}
 		if (n == 0) {
 			/* socket got closed while we were waiting */
 			evutil_closesocket(fd);
-			pxy_conn_ctx_free(ctx);
+			pxy_conn_ctx_free(ctx, 1);
 			return;
 		}
 
@@ -2248,7 +2327,7 @@ pxy_fd_readcb(MAYBE_UNUSED evutil_socket_t fd, UNUSED short what, void *arg)
 			               "ClientHello message, "
 			               "aborting connection\n");
 			evutil_closesocket(fd);
-			pxy_conn_ctx_free(ctx);
+			pxy_conn_ctx_free(ctx, 1);
 			return;
 		}
 		if (OPTS_DEBUG(ctx->opts)) {
@@ -2276,7 +2355,7 @@ pxy_fd_readcb(MAYBE_UNUSED evutil_socket_t fd, UNUSED short what, void *arg)
 				               "event, aborting "
 				               "connection\n");
 				evutil_closesocket(fd);
-				pxy_conn_ctx_free(ctx);
+				pxy_conn_ctx_free(ctx, 1);
 				return;
 			}
 			event_add(ctx->ev, &retry_delay);
@@ -2344,7 +2423,7 @@ pxy_conn_setup(evutil_socket_t fd,
 			log_err_printf("Connection not found in NAT "
 			               "state table, aborting connection\n");
 			evutil_closesocket(fd);
-			pxy_conn_ctx_free(ctx);
+			pxy_conn_ctx_free(ctx, 1);
 			return;
 		}
 	} else if (spec->connect_addrlen > 0) {
@@ -2358,7 +2437,7 @@ pxy_conn_setup(evutil_socket_t fd,
 			log_err_printf("SNI mode used for non-SSL connection; "
 			               "aborting connection\n");
 			evutil_closesocket(fd);
-			pxy_conn_ctx_free(ctx);
+			pxy_conn_ctx_free(ctx, 1);
 			return;
 		}
 	}
@@ -2392,7 +2471,7 @@ pxy_conn_setup(evutil_socket_t fd,
 memout:
 	log_err_printf("Aborting connection setup (out of memory)!\n");
 	evutil_closesocket(fd);
-	pxy_conn_ctx_free(ctx);
+	pxy_conn_ctx_free(ctx, 1);
 	return;
 }
 
