@@ -1,28 +1,29 @@
-/*
+/*-
  * SSLsplit - transparent SSL/TLS interception
- * Copyright (c) 2009-2016, Daniel Roethlisberger <daniel@roe.ch>
+ * https://www.roe.ch/SSLsplit
+ *
+ * Copyright (c) 2009-2018, Daniel Roethlisberger <daniel@roe.ch>.
  * All rights reserved.
- * http://www.roe.ch/SSLsplit
  *
  * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions, and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
+ * modification, are permitted provided that the following conditions are met:
+ * 1. Redistributions of source code must retain the above copyright notice,
+ *    this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright notice,
+ *    this list of conditions and the following disclaimer in the documentation
+ *    and/or other materials provided with the distribution.
  *
- * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
- * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
- * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
- * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
- * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
- * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER AND CONTRIBUTORS ``AS IS''
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
  */
 
 #include "ssl.h"
@@ -88,6 +89,38 @@ ssl_ssl_cert_get(SSL *s)
 }
 #endif /* OpenSSL 0.9.8y, 1.0.0k or 1.0.1e */
 
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
+int
+DH_set0_pqg(DH *dh, BIGNUM *p, BIGNUM *q, BIGNUM *g)
+{
+    /* If the fields p and g in d are NULL, the corresponding input
+     * parameters MUST be non-NULL.  q may remain NULL.
+     */
+    if ((dh->p == NULL && p == NULL)
+        || (dh->g == NULL && g == NULL))
+        return 0;
+
+    if (p != NULL) {
+        BN_free(dh->p);
+        dh->p = p;
+    }
+    if (q != NULL) {
+        BN_free(dh->q);
+        dh->q = q;
+    }
+    if (g != NULL) {
+        BN_free(dh->g);
+        dh->g = g;
+    }
+
+    if (q != NULL) {
+        dh->length = BN_num_bits(q);
+    }
+
+    return 1;
+}
+#endif
+
 
 /*
  * Print OpenSSL version and build-time configuration to standard error and
@@ -153,36 +186,41 @@ ssl_openssl_version(void)
 	                SSL_PROTO_SUPPORT_S);
 
 	fprintf(stderr, "SSL/TLS algorithm availability:");
+#ifndef OPENSSL_NO_SHA0
+	fprintf(stderr, " SHA0");
+#else /* !OPENSSL_NO_SHA0 */
+	fprintf(stderr, " !SHA0");
+#endif /* !OPENSSL_NO_SHA0 */
 #ifndef OPENSSL_NO_RSA
 	fprintf(stderr, " RSA");
-#else /* OPENSSL_NO_RSA */
+#else /* !OPENSSL_NO_RSA */
 	fprintf(stderr, " !RSA");
-#endif /* OPENSSL_NO_RSA */
+#endif /* !OPENSSL_NO_RSA */
 #ifndef OPENSSL_NO_DSA
 	fprintf(stderr, " DSA");
-#else /* OPENSSL_NO_DSA */
+#else /* !OPENSSL_NO_DSA */
 	fprintf(stderr, " !DSA");
-#endif /* OPENSSL_NO_DSA */
+#endif /* !OPENSSL_NO_DSA */
 #ifndef OPENSSL_NO_ECDSA
 	fprintf(stderr, " ECDSA");
-#else /* OPENSSL_NO_ECDSA */
+#else /* !OPENSSL_NO_ECDSA */
 	fprintf(stderr, " !ECDSA");
-#endif /* OPENSSL_NO_ECDSA */
+#endif /* !OPENSSL_NO_ECDSA */
 #ifndef OPENSSL_NO_DH
 	fprintf(stderr, " DH");
-#else /* OPENSSL_NO_DH */
+#else /* !OPENSSL_NO_DH */
 	fprintf(stderr, " !DH");
-#endif /* OPENSSL_NO_DH */
+#endif /* !OPENSSL_NO_DH */
 #ifndef OPENSSL_NO_ECDH
 	fprintf(stderr, " ECDH");
-#else /* OPENSSL_NO_ECDH */
+#else /* !OPENSSL_NO_ECDH */
 	fprintf(stderr, " !ECDH");
-#endif /* OPENSSL_NO_ECDH */
+#endif /* !OPENSSL_NO_ECDH */
 #ifndef OPENSSL_NO_EC
 	fprintf(stderr, " EC");
-#else /* OPENSSL_NO_EC */
+#else /* !OPENSSL_NO_EC */
 	fprintf(stderr, " !EC");
-#endif /* OPENSSL_NO_EC */
+#endif /* !OPENSSL_NO_EC */
 	fprintf(stderr, "\n");
 
 	fprintf(stderr, "OpenSSL option availability:");
@@ -226,7 +264,7 @@ ssl_openssl_version(void)
  */
 static int ssl_initialized = 0;
 
-#ifdef OPENSSL_THREADS
+#if defined(OPENSSL_THREADS) && OPENSSL_VERSION_NUMBER < 0x10100000L
 struct CRYPTO_dynlock_value {
 	pthread_mutex_t mutex;
 };
@@ -256,7 +294,10 @@ ssl_thr_dyn_create_cb(UNUSED const char *file, UNUSED int line)
 	struct CRYPTO_dynlock_value *dl;
 
 	if ((dl = malloc(sizeof(struct CRYPTO_dynlock_value)))) {
-		pthread_mutex_init(&dl->mutex, NULL);
+		if (pthread_mutex_init(&dl->mutex, NULL)) {
+			free(dl);
+			return NULL;
+		}
 	}
 	return dl;
 }
@@ -331,11 +372,14 @@ ssl_init(void)
 	OpenSSL_add_all_algorithms();
 
 	/* thread-safety */
-#ifdef OPENSSL_THREADS
+#if defined(OPENSSL_THREADS) && OPENSSL_VERSION_NUMBER < 0x10100000L
 	ssl_mutex_num = CRYPTO_num_locks();
 	ssl_mutex = malloc(ssl_mutex_num * sizeof(*ssl_mutex));
 	for (int i = 0; i < ssl_mutex_num; i++) {
-		pthread_mutex_init(&ssl_mutex[i], NULL);
+		if (pthread_mutex_init(&ssl_mutex[i], NULL)) {
+			log_err_printf("Failed to initialize mutex\n");
+			return -1;
+		}
 	}
 	CRYPTO_set_locking_callback(ssl_thr_locking_cb);
 	CRYPTO_set_dynlock_create_callback(ssl_thr_dyn_create_cb);
@@ -389,19 +433,23 @@ ssl_init(void)
 }
 
 /*
- * Re-initialize OpenSSL after forking.
+ * Re-initialize OpenSSL after forking.  Returns 0 on success, -1 on failure.
  */
-void
+int
 ssl_reinit(void)
 {
 	if (!ssl_initialized)
-		return;
+		return 0;
 
-#ifdef OPENSSL_THREADS
+#if defined(OPENSSL_THREADS) && OPENSSL_VERSION_NUMBER < 0x10100000L
 	for (int i = 0; i < ssl_mutex_num; i++) {
-		pthread_mutex_init(&ssl_mutex[i], NULL);
+		if (pthread_mutex_init(&ssl_mutex[i], NULL)) {
+			return -1;
+		}
 	}
 #endif /* OPENSSL_THREADS */
+
+	return 0;
 }
 
 /*
@@ -414,9 +462,11 @@ ssl_fini(void)
 	if (!ssl_initialized)
 		return;
 
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
 	ERR_remove_state(0); /* current thread */
+#endif
 
-#ifdef OPENSSL_THREADS
+#if defined(OPENSSL_THREADS) && OPENSSL_VERSION_NUMBER < 0x10100000L
 	CRYPTO_set_locking_callback(NULL);
 	CRYPTO_set_dynlock_create_callback(NULL);
 	CRYPTO_set_dynlock_lock_callback(NULL);
@@ -476,16 +526,67 @@ ssl_ssl_state_to_str(SSL *ssl)
 	char *str = NULL;
 	int rv;
 
-	rv = asprintf(&str, "%08x = %s%s%s%04x = %s (%s) [%s]",
-	              ssl->state,
-	              (ssl->state & SSL_ST_CONNECT) ? "SSL_ST_CONNECT|" : "",
-	              (ssl->state & SSL_ST_ACCEPT) ? "SSL_ST_ACCEPT|" : "",
-	              (ssl->state & SSL_ST_BEFORE) ? "SSL_ST_BEFORE|" : "",
-	              ssl->state & SSL_ST_MASK,
+	rv = asprintf(&str, "%08x = %s%s%04x = %s (%s) [%s]",
+	              SSL_get_state(ssl),
+	              (SSL_get_state(ssl) & SSL_ST_CONNECT) ? "SSL_ST_CONNECT|" : "",
+	              (SSL_get_state(ssl) & SSL_ST_ACCEPT) ? "SSL_ST_ACCEPT|" : "",
+	              SSL_get_state(ssl) & SSL_ST_MASK,
 	              SSL_state_string(ssl),
 	              SSL_state_string_long(ssl),
-	              (ssl->type == SSL_ST_CONNECT) ? "connect socket"
-	                                            : "accept socket");
+	              SSL_is_server(ssl) ? "accept socket" : "connect socket");
+
+	return (rv < 0) ? NULL : str;
+}
+
+/*
+ * Generates a NSS key log format compatible string containing the client
+ * random and the master key, intended to be used to decrypt externally
+ * captured network traffic using tools like Wireshark.
+ *
+ * Only supports the CLIENT_RANDOM method (SSL 3.0 - TLS 1.2).
+ *
+ * https://developer.mozilla.org/en-US/docs/Mozilla/Projects/NSS/Key_Log_Format
+ */
+char *
+ssl_ssl_masterkey_to_str(SSL *ssl)
+{
+	char *str = NULL;
+	int rv;
+	unsigned char *k, *r;
+#if OPENSSL_VERSION_NUMBER >= 0x10100000L
+	unsigned char kbuf[48], rbuf[32];
+	k = &kbuf[0];
+	r = &rbuf[0];
+	SSL_SESSION_get_master_key(SSL_get0_session(ssl), k, sizeof(kbuf));
+	SSL_get_client_random(ssl, r, sizeof(rbuf));
+#else /* OPENSSL_VERSION_NUMBER < 0x10100000L */
+	k = ssl->session->master_key;
+	r = ssl->s3->client_random;
+#endif /* OPENSSL_VERSION_NUMBER < 0x10100000L */
+	rv = asprintf(&str,
+	              "CLIENT_RANDOM "
+	              "%02X%02X%02X%02X%02X%02X%02X%02X"
+	              "%02X%02X%02X%02X%02X%02X%02X%02X"
+	              "%02X%02X%02X%02X%02X%02X%02X%02X"
+	              "%02X%02X%02X%02X%02X%02X%02X%02X"
+	              " "
+	              "%02X%02X%02X%02X%02X%02X%02X%02X"
+	              "%02X%02X%02X%02X%02X%02X%02X%02X"
+	              "%02X%02X%02X%02X%02X%02X%02X%02X"
+	              "%02X%02X%02X%02X%02X%02X%02X%02X"
+	              "%02X%02X%02X%02X%02X%02X%02X%02X"
+	              "%02X%02X%02X%02X%02X%02X%02X%02X"
+	              "\n",
+	              r[ 0], r[ 1], r[ 2], r[ 3], r[ 4], r[ 5], r[ 6], r[ 7],
+	              r[ 8], r[ 9], r[10], r[11], r[12], r[13], r[14], r[15],
+	              r[16], r[17], r[18], r[19], r[20], r[21], r[22], r[23],
+	              r[24], r[25], r[26], r[27], r[28], r[29], r[30], r[31],
+	              k[ 0], k[ 1], k[ 2], k[ 3], k[ 4], k[ 5], k[ 6], k[ 7],
+	              k[ 8], k[ 9], k[10], k[11], k[12], k[13], k[14], k[15],
+	              k[16], k[17], k[18], k[19], k[20], k[21], k[22], k[23],
+	              k[24], k[25], k[26], k[27], k[28], k[29], k[30], k[31],
+	              k[32], k[33], k[34], k[35], k[36], k[37], k[38], k[39],
+	              k[40], k[41], k[42], k[43], k[44], k[45], k[46], k[47]);
 
 	return (rv < 0) ? NULL : str;
 }
@@ -587,38 +688,49 @@ DH *
 ssl_tmp_dh_callback(UNUSED SSL *s, int is_export, int keylength)
 {
 	DH *dh;
+	int rv = 0;
 
 	if (!(dh = DH_new())) {
 		log_err_printf("DH_new() failed\n");
 		return NULL;
 	}
 	switch (keylength) {
-		case 512:
-			dh->p = BN_bin2bn(dh512_p, sizeof(dh512_p), NULL);
-			break;
-		case 1024:
-			dh->p = BN_bin2bn(dh1024_p, sizeof(dh1024_p), NULL);
-			break;
-		case 2048:
-			dh->p = BN_bin2bn(dh2048_p, sizeof(dh2048_p), NULL);
-			break;
-		case 4096:
-			dh->p = BN_bin2bn(dh4096_p, sizeof(dh4096_p), NULL);
-			break;
-		default:
-			log_err_printf("Unhandled DH keylength %i%s\n",
-			               keylength,
-			               (is_export ? " (export)" : ""));
-			DH_free(dh);
-			return NULL;
+	case 512:
+		rv = DH_set0_pqg(dh,
+		                 BN_bin2bn(dh512_p, sizeof(dh512_p), NULL),
+		                 NULL,
+		                 BN_bin2bn(dh_g, sizeof(dh_g), NULL));
+		break;
+	case 1024:
+		rv = DH_set0_pqg(dh,
+		                 BN_bin2bn(dh1024_p, sizeof(dh1024_p), NULL),
+		                 NULL,
+		                 BN_bin2bn(dh_g, sizeof(dh_g), NULL));
+		break;
+	case 2048:
+		rv = DH_set0_pqg(dh,
+		                 BN_bin2bn(dh2048_p, sizeof(dh2048_p), NULL),
+		                 NULL,
+		                 BN_bin2bn(dh_g, sizeof(dh_g), NULL));
+		break;
+	case 4096:
+		rv = DH_set0_pqg(dh,
+		                 BN_bin2bn(dh4096_p, sizeof(dh4096_p), NULL),
+		                 NULL,
+		                 BN_bin2bn(dh_g, sizeof(dh_g), NULL));
+		break;
+	default:
+		log_err_printf("Unhandled DH keylength %i%s\n",
+		               keylength,
+		               (is_export ? " (export)" : ""));
+		DH_free(dh);
+		return NULL;
 	}
-	dh->g = BN_bin2bn(dh_g, sizeof(dh_g), NULL);
-	if (!dh->p || !dh->g) {
+	if (!rv) {
 		log_err_printf("Failed to load DH p and g from memory\n");
 		DH_free(dh);
 		return NULL;
 	}
-
 	return(dh);
 }
 
@@ -717,13 +829,15 @@ ssl_rand(void *p, size_t sz)
 {
 	int rv;
 
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
 	rv = RAND_pseudo_bytes((unsigned char*)p, sz);
-	if (rv == -1) {
-		rv = RAND_bytes((unsigned char*)p, sz);
-		if (rv != 1)
-			return -1;
-	}
-	return 0;
+	if (rv == 1)
+		return 0;
+#endif /* OPENSSL_VERSION_NUMBER < 0x10100000L */
+	rv = RAND_bytes((unsigned char*)p, sz);
+	if (rv == 1)
+		return 0;
+	return -1;
 }
 
 /*
@@ -774,13 +888,14 @@ ssl_x509_serial_copyrand(X509 *dstcrt, X509 *srccrt)
  * The optional argument extraname is added to subjectAltNames if provided.
  */
 X509 *
-ssl_x509_forge(X509 *cacrt, EVP_PKEY *cakey, X509 *origcrt,
-               const char *extraname, EVP_PKEY *key)
+ssl_x509_forge(X509 *cacrt, EVP_PKEY *cakey, X509 *origcrt, EVP_PKEY *key,
+               const char *extraname, const char *crlurl)
 {
 	X509_NAME *subject, *issuer;
 	GENERAL_NAMES *names;
 	GENERAL_NAME *gn;
 	X509 *crt;
+	int rv;
 
 	subject = X509_get_subject_name(origcrt);
 	issuer = X509_get_subject_name(cacrt);
@@ -801,21 +916,52 @@ ssl_x509_forge(X509 *cacrt, EVP_PKEY *cakey, X509 *origcrt,
 		goto errout;
 
 	/* add standard v3 extensions; cf. RFC 2459 */
+
 	X509V3_CTX ctx;
 	X509V3_set_ctx(&ctx, cacrt, crt, NULL, NULL, 0);
-	if (ssl_x509_v3ext_add(&ctx, crt, "basicConstraints",
-	                                  "CA:FALSE") == -1 ||
-	    ssl_x509_v3ext_add(&ctx, crt, "keyUsage",
-	                                  "digitalSignature,"
-	                                  "keyEncipherment") == -1 ||
-	    ssl_x509_v3ext_add(&ctx, crt, "extendedKeyUsage",
-	                                  "serverAuth") == -1 ||
-	    ssl_x509_v3ext_add(&ctx, crt, "subjectKeyIdentifier",
+	if (ssl_x509_v3ext_add(&ctx, crt, "subjectKeyIdentifier",
 	                                  "hash") == -1 ||
 	    ssl_x509_v3ext_add(&ctx, crt, "authorityKeyIdentifier",
 	                                  "keyid,issuer:always") == -1)
 		goto errout;
 
+	rv = ssl_x509_v3ext_copy_by_nid(crt, origcrt,
+	                                NID_basic_constraints);
+	if (rv == 0)
+		rv = ssl_x509_v3ext_add(&ctx, crt, "basicConstraints",
+		                                   "CA:FALSE");
+	if (rv == -1)
+		goto errout;
+
+	rv = ssl_x509_v3ext_copy_by_nid(crt, origcrt,
+	                                NID_key_usage);
+	if (rv == 0)
+		rv = ssl_x509_v3ext_add(&ctx, crt, "keyUsage",
+		                                   "digitalSignature,"
+		                                   "keyEncipherment");
+	if (rv == -1)
+		goto errout;
+
+	rv = ssl_x509_v3ext_copy_by_nid(crt, origcrt,
+	                                NID_ext_key_usage);
+	if (rv == 0)
+		rv = ssl_x509_v3ext_add(&ctx, crt, "extendedKeyUsage",
+		                                   "serverAuth");
+	if (rv == -1)
+		goto errout;
+
+	if (crlurl) {
+		char *crlurlval;
+		if (asprintf(&crlurlval, "URI:%s", crlurl) < 0)
+			goto errout;
+		if (ssl_x509_v3ext_add(&ctx, crt, "crlDistributionPoints",
+		                       crlurlval) == -1) {
+			free(crlurlval);
+			goto errout;
+		}
+		free(crlurlval);
+	}
+
 	if (!extraname) {
 		/* no extraname provided: copy original subjectAltName ext */
 		if (ssl_x509_v3ext_copy_by_nid(crt, origcrt,
@@ -841,7 +987,7 @@ ssl_x509_forge(X509 *cacrt, EVP_PKEY *cakey, X509 *origcrt,
 			if (!gn)
 				goto errout2;
 			gn->type = GEN_DNS;
-			gn->d.dNSName = M_ASN1_IA5STRING_new();
+			gn->d.dNSName = ASN1_IA5STRING_new();
 			if (!gn->d.dNSName)
 				goto errout3;
 			ASN1_STRING_set(gn->d.dNSName,
@@ -865,10 +1011,10 @@ ssl_x509_forge(X509 *cacrt, EVP_PKEY *cakey, X509 *origcrt,
 #endif /* DEBUG_CERTIFICATE */
 
 	const EVP_MD *md;
-	switch (EVP_PKEY_type(cakey->type)) {
+	switch (EVP_PKEY_type(EVP_PKEY_base_id(cakey))) {
 #ifndef OPENSSL_NO_RSA
 	case EVP_PKEY_RSA:
-		switch (OBJ_obj2nid(origcrt->sig_alg->algorithm)) {
+		switch (X509_get_signature_nid(origcrt)) {
 		case NID_md5WithRSAEncryption:
 			md = EVP_md5();
 			break;
@@ -887,7 +1033,11 @@ ssl_x509_forge(X509 *cacrt, EVP_PKEY *cakey, X509 *origcrt,
 		case NID_sha512WithRSAEncryption:
 			md = EVP_sha512();
 			break;
+#ifndef OPENSSL_NO_SHA0
 		case NID_shaWithRSAEncryption:
+			md = EVP_sha();
+			break;
+#endif /* !OPENSSL_NO_SHA0 */
 		case NID_sha1WithRSAEncryption:
 		default:
 			md = EVP_sha1();
@@ -897,12 +1047,46 @@ ssl_x509_forge(X509 *cacrt, EVP_PKEY *cakey, X509 *origcrt,
 #endif /* !OPENSSL_NO_RSA */
 #ifndef OPENSSL_NO_DSA
 	case EVP_PKEY_DSA:
-		md = EVP_dss1();
+		switch (X509_get_signature_nid(origcrt)) {
+		case NID_dsa_with_SHA224:
+			md = EVP_sha224();
+			break;
+		case NID_dsa_with_SHA256:
+			md = EVP_sha256();
+			break;
+#ifndef OPENSSL_NO_SHA0
+		case NID_dsaWithSHA:
+			md = EVP_sha();
+			break;
+#endif /* !OPENSSL_NO_SHA0 */
+		case NID_dsaWithSHA1:
+		case NID_dsaWithSHA1_2:
+		default:
+			md = EVP_sha1();
+			break;
+		}
 		break;
 #endif /* !OPENSSL_NO_DSA */
 #ifndef OPENSSL_NO_ECDSA
 	case EVP_PKEY_EC:
-		md = EVP_ecdsa();
+		switch (X509_get_signature_nid(origcrt)) {
+		case NID_ecdsa_with_SHA224:
+			md = EVP_sha224();
+			break;
+		case NID_ecdsa_with_SHA256:
+			md = EVP_sha256();
+			break;
+		case NID_ecdsa_with_SHA384:
+			md = EVP_sha384();
+			break;
+		case NID_ecdsa_with_SHA512:
+			md = EVP_sha512();
+			break;
+		case NID_ecdsa_with_SHA1:
+		default:
+			md = EVP_sha1();
+			break;
+		}
 		break;
 #endif /* !OPENSSL_NO_ECDSA */
 	default:
@@ -1015,7 +1199,6 @@ ssl_x509chain_use(SSL_CTX *sslctx, X509 *crt, STACK_OF(X509) *chain)
 
 		tmpcrt = sk_X509_value(chain, i);
 		ssl_x509_refcount_inc(tmpcrt);
-		sk_X509_push(sslctx->extra_certs, tmpcrt);
 		SSL_CTX_add_extra_chain_cert(sslctx, tmpcrt);
 	}
 }
@@ -1097,12 +1280,26 @@ leave1:
 EVP_PKEY *
 ssl_key_genrsa(const int keysize)
 {
-	EVP_PKEY * pkey;
-	RSA * rsa;
+	EVP_PKEY *pkey;
+	RSA *rsa;
 
+#if OPENSSL_VERSION_NUMBER >= 0x10100000L
+	BIGNUM *bn;
+	int rv;
+	rsa = RSA_new();
+	bn = BN_new();
+	BN_dec2bn(&bn, "3");
+	rv = RSA_generate_key_ex(rsa, keysize, bn, NULL);
+	BN_free(bn);
+	if (rv != 1) {
+		RSA_free(rsa);
+		return NULL;
+	}
+#else /* OPENSSL_VERSION_NUMBER < 0x10100000L */
 	rsa = RSA_generate_key(keysize, 3, NULL, NULL);
 	if (!rsa)
 		return NULL;
+#endif /* OPENSSL_VERSION_NUMBER < 0x10100000L */
 	pkey = EVP_PKEY_new();
 	EVP_PKEY_assign_RSA(pkey, rsa); /* does not increment refcount */
 	return pkey;
@@ -1117,14 +1314,15 @@ int
 ssl_key_identifier_sha1(EVP_PKEY *key, unsigned char *keyid)
 {
 	X509_PUBKEY *pubkey = NULL;
-	ASN1_BIT_STRING *pk;
+	const unsigned char *pk;
+	int length;
 
 	/* X509_PUBKEY_set() will attempt to free pubkey if != NULL */
 	if (X509_PUBKEY_set(&pubkey, key) != 1 || !pubkey)
 		return -1;
-	if (!(pk = pubkey->public_key))
+	if (!X509_PUBKEY_get0_param(NULL, &pk, &length, NULL, pubkey))
 		goto errout;
-	if (!EVP_Digest(pk->data, pk->length, keyid, NULL, EVP_sha1(), NULL))
+	if (!EVP_Digest(pk, length, keyid, NULL, EVP_sha1(), NULL))
 		goto errout;
 	X509_PUBKEY_free(pubkey);
 	return 0;
@@ -1221,10 +1419,10 @@ ssl_x509_fingerprint(X509 *crt, int colons)
 void
 ssl_dh_refcount_inc(DH *dh)
 {
-#ifdef OPENSSL_THREADS
+#if defined(OPENSSL_THREADS) && OPENSSL_VERSION_NUMBER < 0x10100000L
 	CRYPTO_add(&dh->references, 1, CRYPTO_LOCK_DH);
 #else /* !OPENSSL_THREADS */
-	dh->references++;
+	DH_up_ref(dh);
 #endif /* !OPENSSL_THREADS */
 }
 #endif /* !OPENSSL_NO_DH */
@@ -1236,10 +1434,10 @@ ssl_dh_refcount_inc(DH *dh)
 void
 ssl_key_refcount_inc(EVP_PKEY *key)
 {
-#ifdef OPENSSL_THREADS
+#if defined(OPENSSL_THREADS) && OPENSSL_VERSION_NUMBER < 0x10100000L
 	CRYPTO_add(&key->references, 1, CRYPTO_LOCK_EVP_PKEY);
 #else /* !OPENSSL_THREADS */
-	key->references++;
+	EVP_PKEY_up_ref(key);
 #endif /* !OPENSSL_THREADS */
 }
 
@@ -1251,10 +1449,10 @@ ssl_key_refcount_inc(EVP_PKEY *key)
 void
 ssl_x509_refcount_inc(X509 *crt)
 {
-#ifdef OPENSSL_THREADS
+#if defined(OPENSSL_THREADS) && OPENSSL_VERSION_NUMBER < 0x10100000L
 	CRYPTO_add(&crt->references, 1, CRYPTO_LOCK_X509);
 #else /* !OPENSSL_THREADS */
-	crt->references++;
+	X509_up_ref(crt);
 #endif /* !OPENSSL_THREADS */
 }
 
@@ -1325,8 +1523,10 @@ ssl_wildcardify(const char *hostname)
 	if (!(wildcarded = malloc(dotsz + 2)))
 		return NULL;
 	wildcarded[0] = '*';
-	strncpy(wildcarded + 1, dot, dotsz);
-	wildcarded[dotsz + 1] = '\0';
+	for (size_t i = 0; i < dotsz; i++) {
+		wildcarded[i+1] = dot[i];
+	}
+	wildcarded[dotsz+1] = '\0';
 	return wildcarded;
 }
 
@@ -1448,6 +1648,7 @@ ssl_x509_names(X509 *crt)
  * in the Subject DN CN and subjectAltNames extension, separated by slashes.
  * Caller must free returned buffer.
  * Embedded NULL characters in hostnames are replaced with '!'.
+ * If no CN and no subjectAltNames are found, returns "-".
  */
 char *
 ssl_x509_names_to_str(X509 *crt)
@@ -1458,7 +1659,7 @@ ssl_x509_names_to_str(X509 *crt)
 
 	names = ssl_x509_names(crt);
 	if (!names)
-		return NULL;
+		return strdup("-");
 
 	sz = 0;
 	for (char **p = names; *p; p++) {
@@ -1726,10 +1927,11 @@ ssl_is_ocspreq(const unsigned char *buf, size_t sz)
  * message beginning at offsets >= 0, whereas if search is zero, only
  * ClientHello messages starting at offset 0 will be considered.
  *
- * Note that this code currently only supports SSL 3.0 and TLS 1.0-1.2 and that
- * it expects the ClientHello message to be unfragmented in a single record.
+ * This code currently supports SSL 2.0, SSL 3.0 and TLS 1.0-1.2.
  *
  * References:
+ * draft-hickman-netscape-ssl-00: The SSL Protocol
+ * RFC 6101: The Secure Sockets Layer (SSL) Protocol Version 3.0
  * RFC 2246: The TLS Protocol Version 1.0
  * RFC 3546: Transport Layer Security (TLS) Extensions
  * RFC 4346: The Transport Layer Security (TLS) Protocol Version 1.1
@@ -1770,8 +1972,8 @@ ssl_tls_clienthello_parse(const unsigned char *buf, ssize_t sz, int search,
 		}
 
 		if (search) {
-			/* Search for the beginning of a potential ClientHello */
-			while ((n > 0) && (*p != 22)) {
+			/* Search for a potential ClientHello */
+			while ((n > 0) && (*p != 0x16) && (*p != 0x80)) {
 				p++; n--;
 			}
 			if (n <= 0) {
@@ -1779,7 +1981,8 @@ ssl_tls_clienthello_parse(const unsigned char *buf, ssize_t sz, int search,
 				 * clienthello to NULL to indicate to the
 				 * caller that this buffer does not need to be
 				 * retried */
-				DBG_printf("===> No match: rv 1, *clienthello NULL\n");
+				DBG_printf("===> No match:"
+				           " rv 1, *clienthello NULL\n");
 				*clienthello = NULL;
 				return 1;
 			}
@@ -1788,9 +1991,68 @@ ssl_tls_clienthello_parse(const unsigned char *buf, ssize_t sz, int search,
 		DBG_printf("candidate at offset %td\n", p - buf);
 
 		DBG_printf("byte 0: %02x\n", *p);
-		/* +0 0x80 +2 0x01 SSLv2 clientHello;
-		 * +0 0x22 +1 0x03 SSLv3/TLSv1.x clientHello */
-		if (*p != 22) { /* record type: handshake protocol */
+		/* +0 0x80 +2 0x01 SSLv2 short header, clientHello;
+		 * +0 0x16 +1 0x03 SSLv3/TLSv1.x handshake, clientHello */
+		if (*p == 0x80) {
+			/* SSLv2 handled here */
+			p++; n--;
+
+			if (n < 10) { /* length + 9 */
+				DBG_printf("===> [SSLv2] Truncated:"
+				           " rv 1, *clienthello set\n");
+				return 1;
+			}
+
+			DBG_printf("length: %02x\n", p[0]);
+			if (n - 1 < p[0]) {
+				DBG_printf("===> [SSLv2] Truncated:"
+				           " rv 1, *clienthello set\n");
+				return 1;
+			}
+			p++; n--;
+
+			DBG_printf("msgtype: %02x\n", p[0]);
+			if (*p != 0x01)
+				continue;
+			p++; n--;
+
+			DBG_printf("version: %02x %02x\n", p[0], p[1]);
+			/* byte order is actually swapped for SSLv2 */
+			if (!(
+#ifdef HAVE_SSLV2
+			      (p[0] == 0x00 && p[1] == 0x02) ||
+#endif /* HAVE_SSLV2 */
+			      (p[0] == 0x03 && p[1] <= 0x03)))
+				continue;
+			p += 2; n -= 2;
+
+			DBG_printf("cipher-spec-len: %02x %02x\n", p[0], p[1]);
+			ssize_t cipherspec_len = p[0] << 8 | p[1];
+			p += 2; n -= 2;
+
+			DBG_printf("session-id-len: %02x %02x\n", p[0], p[1]);
+			ssize_t sessionid_len = p[0] << 8 | p[1];
+			p += 2; n -= 2;
+
+			DBG_printf("challenge-len: %02x %02x\n", p[0], p[1]);
+			ssize_t challenge_len = p[0] << 8 | p[1];
+			p += 2; n -= 2;
+			if (challenge_len < 16 || challenge_len > 32)
+				continue;
+
+			if (n < cipherspec_len
+			      + sessionid_len
+			      + challenge_len) {
+				DBG_printf("===> [SSLv2] Truncated:"
+				           " rv 1, *clienthello set\n");
+				return 1;
+			}
+
+			p += cipherspec_len + sessionid_len + challenge_len;
+			n -= cipherspec_len + sessionid_len + challenge_len;
+			goto done_parsing;
+		} else
+		if (*p != 0x16) {
 			/* this can only happen if search is 0 */
 			DBG_printf("===> No match: rv 1, *clienthello NULL\n");
 			*clienthello = NULL;
@@ -1807,7 +2069,7 @@ ssl_tls_clienthello_parse(const unsigned char *buf, ssize_t sz, int search,
 		 * updated for TLS 1.3 once that is standardized and still
 		 * compatible with this parser; remember to also update the
 		 * inner version check below */
-		if (p[0] != 0x03 && p[1] > 0x03)
+		if (p[0] != 0x03 || p[1] > 0x03)
 			continue;
 		p += 2; n -= 2;
 
@@ -1988,6 +2250,8 @@ ssl_tls_clienthello_parse(const unsigned char *buf, ssize_t sz, int search,
 			n -= extlen;
 		} /* while have more extensions */
 
+done_parsing:
+		;
 #ifdef DEBUG_CLIENTHELLO_PARSER
 		if (n > 0) {
 			DBG_printf("unparsed next bytes %02x %02x %02x %02x\n",
@@ -2002,7 +2266,7 @@ ssl_tls_clienthello_parse(const unsigned char *buf, ssize_t sz, int search,
 			*servername = sn;
 		return 0;
 continue_search:
-	;
+		;
 	} while (search && n > 0);
 
 	/* No valid ClientHello messages found, not even a truncated one */
